<?php

namespace app\admin\controller;

use app\admin\model\ActionLog;
use app\admin\model\OrderImportFromC as OIFCModel;
use app\admin\model\ProcessOrder;
use Exception;
use think\facade\Db;
use think\facade\Validate;

//C端订单导入及处理类
class OrderImport extends Base
{

    //订单导入
    public function import()
    {

        $param = $this->request->only(['token', 'list'], 'post', 'trim');

        $val_params = Validate::rule(['token' => 'require', 'list' => 'require|array']);

        if (!$val_params->check($param)) throw new \think\Exception($val_params->getError());

        Db::startTrans();

        try {

            $userinfo = GetUserInfo($param['token']);
            $createrid = isset($user["data"]['id']) ? $userinfo["data"]['id'] : 0;
            $creater = isset($user["data"]['nickname']) ? $userinfo["data"]['nickname'] : "";

            $insert_data = [];
            foreach ($param['list'] as $key => $value) {
                $value['createrid'] = $createrid;
                $value['creater'] = $creater;
                $insert_data[] = $value;
            }

            $res = Db::name('order_import_from_c')
                ->limit(100)
                ->insertAll($insert_data);

            if ($res) {
                Db::commit();
                return app_show(0, "导入成功，共" . (string)$res . '条记录');
            } else {
                Db::rollback();

                return error_show(1005, '导入失败');
            }

        } catch (Exception $exception) {
            Db::rollback();
            return error_show(1005, $exception->getMessage());
        }

    }

    //查看订单录入列表
    public function getImportList()
    {
        $param = $this->request->only([
            'status' => 0,
            'platform_code' => '',
            'po_code' => '',
            'platform_time_start' => '',
            'platform_time_end' => '',
            'addtime_start' => '',
            'addtime_end' => '',
            'plat_code' => '',
            'page' => 1,
            'size' => 15,
        ], 'post', 'trim');

        $where = [];
        if ($param['status']) $where[] = ['c.status', '=', $param['status']];
        if ($param['platform_code']) $where[] = ['c.platform_code', 'like', '%' . $param['platform_code'] . '%'];
        if ($param['po_code']) $where[] = ['c.po_code', 'like', '%' . $param['po_code'] . '%'];
        if ($param['platform_time_start'] && $param['platform_time_end']) $where[] = ['c.platform_time', 'between', [$param['platform_time_start'], $param['platform_time_end']]];
        if ($param['addtime_start'] && $param['addtime_end']) $where[] = ['c.addtime', 'between', [$param['addtime_start'], $param['addtime_end']]];
        if ($param['plat_code']) $where[] = ['c.plat_code', 'like', '%' . $param['plat_code'] . '%'];

        $db = OIFCModel::alias('c')->where($where);

        $total = $db->count('id');

        $list = $db
            ->page($param['page'], $param['size'])
            ->order('id', 'desc')
            ->select()
            ->toArray();

        return app_show(0, "获取成功", ['list' => $list, 'count' => $total]);
    }

    //查看订单录入详情
    public function getImportInfo()
    {
        $id = $this->request->post('id/d', 0, 'trim');

        $info = OIFCModel::alias('c')
            ->field('c.*,ce.*,b.company,ci.companyName CustomerCompanyName,s.name supplierName')
            ->leftJoin('order_import_from_c_extend ce', 'ce.order_import_from_c_id=c.id')
            ->leftJoin('business b', 'b.companyNo=ce.companyNo AND b.is_del=0')
            ->leftJoin('customer_info ci', 'ci.companyNo=ce.customer_code AND ci.is_del=0')
            ->leftJoin('supplier s', 's.code=ce.supplierNo AND s.is_del=0')
            ->where('c.id', $id)
            ->find();

        if (!empty($info)) {
            $info['platform_names'] = Db::name('platform')
                ->whereIn('platform_code', $info['platform_codes'])
                ->where(['is_del' => 0, 'status' => 1])
                ->column('platform_name');

            $info['addr_name'] = GetAddr($info['addr_code']);

            $res['id'] = $info['id'];
            $res['status'] = $info['status'];
            $res['createrid'] = $info['createrid'];
            $res['creater'] = $info['creater'];
            $res['updateid'] = $info['updateid'];
            $res['updater'] = $info['updater'];

            //导入字段
            $res['import'] = [
                'platform_code' => $info['platform_code'],
                'platform_time' => $info['platform_time'],
                'plat_code' => $info['plat_code'],
                'sale_source' => $info['sale_source'],
                'price' => $info['price'],
                'num' => $info['num'],
                'before_discount_all_price' => $info['before_discount_all_price'],
                'activity_name' => $info['activity_name'],
                'mode' => $info['mode'],
                'order_remark' => $info['order_remark'],
                'contactor' => $info['contactor'],
                'mobile' => $info['mobile'],
                'addr' => $info['addr'],
            ];
            //确认字段
            $res['confirm'] = $info;

            return app_show(0, '请求成功', $res);

        } else  return error_show(1005, '系统尚未解析完成，无法查看详情,请等待');

    }

    //用户确认订单数据(编辑)
    public function confirmImportData()
    {
        //每次编辑，都新增一条记录到extend表，同时原纪录作废，确保同一个order_import_from_c_id下只有一个正常的记录
        $param = $this->request->only([
            'id',
            'platform_code',
            'po_code',
            'platform_time',
            'sale_source',
            'plat_code',
            'price',
            'num',
            'before_discount_all_price',
            'activity_id',
            'activity_name',
            'mode',
            'order_remark',
            'contactor',
            'mobile',
            'addr',
            'addr_code',
        ], 'post', 'trim');

        $val = Validate::rule([
            'id' => 'require',
            'platform_code' => 'require',
            'po_code' => 'require',
            'platform_time' => 'require',
            'sale_source' => 'require',
            'plat_code' => 'require',
            'price' => 'require',
            'num' => 'require',
            'before_discount_all_price' => 'require',
            'mode' => 'require',
            'contactor' => 'require',
            'mobile' => 'require',
            'addr' => 'require',
            'addr_code' => 'require',
        ]);

        if (!$val->check($param)) return error_show(1004, $val->getError());

        Db::startTrans();
        try {

            $rs = Db::name('order_import_from_c')
                ->where(['id' => $param['id'], 'is_del' => OIFCModel::$is_del_normal, 'status' => OIFCModel::$status_wait_confirm])
                ->find();
            if (empty($rs)) throw new \think\Exception('该记录不存在或不允许用户确认');

            $userinfo = GetUserInfo($param['token']);

            Db::name('order_import_from_c')
                ->where(['id' => $param['id'], 'is_del' => OIFCModel::$is_del_normal, 'status' => OIFCModel::$status_wait_confirm])
                ->update([
                    'platform_code' => $param['platform_code'],
                    'po_code' => $param['po_code'],
                    'platform_time' => $param['platform_time'],
                    'sale_source' => $param['sale_source'],
                    'plat_code' => $param['plat_code'],
                    'price' => $param['price'],
                    'num' => $param['num'],
                    'before_discount_all_price' => $param['before_discount_all_price'],
                    'activity_name' => $param['activity_name'],
                    'mode' => $param['mode'],
                    'order_remark' => $param['order_remark'],
                    'contactor' => $param['contactor'],
                    'mobile' => $param['mobile'],
                    'addr' => $param['addr'],
                    'status' => OIFCModel::$status_wait_relation,
                    'updatetime' => date('Y-m-d H:i:s')
                ]);

            Db::name('order_import_from_c_extend')
                ->where(['order_import_from_c_id' => $param['id'], 'is_del' => 0])
                ->update(['is_del' => 1]);

            Db::name('order_import_from_c_extend')
                ->insert([
                    'order_import_from_c_id' => $param['id'],
                    'addr_code' => $param['addr_code'],
                    'activity_id' => $param['activity_id'],
                    'addtime' => date('Y-m-d H:i:s'),
                    'createrid' => isset($user["data"]['id']) ? $userinfo["data"]['id'] : 0,
                    'creater' => isset($user["data"]['nickname']) ? $userinfo["data"]['nickname'] : '',
                ]);

            Db::commit();

            return app_show(0, '修改成功');

        } catch (\think\Exception $exception) {

            Db::rollback();

            return error_show(1004, $exception->getMessage());
        }

    }

    //批量关联库存
    public function relationStockBatch()
    {

        $ids = $this->request->post('ids/a', [], 'trim');

        if (empty($ids)) return error_show(1004, 'ids参数不允许为空');

        $res = Db::name('order_import_from_c')
            ->whereIn('id', $ids)
            ->where(['is_del' => OIFCModel::$is_del_normal, 'status' => OIFCModel::$status_stock_not_enough])
            ->update(['status' => OIFCModel::$status_wait_relation, 'updatetime' => date('Y-m-d H:i:s')]);

        if ($res) return app_show(0, '批量关联库存成功');
        else return error_show(1005, '批量关联库存失败');

    }


    //【脚本2】批量处理导入的数据
    public function analysisBatchDataBySystem2222()
    {
        //解析信息存到extend表,其中customer_code字段根据订单编号去customer_info查，其他根据商品编号查询

        Db::startTrans();
        try {


            $order_import_from_c_db = Db::name('order_import_from_c');

            $order_import_from_c_extend_db = Db::name('order_import_from_c_extend');
//        $customer_info_db = Db::name('customer_info')->field('companyNo')->where(['is_del' => 0]);

            $rs = $order_import_from_c_db
                ->where([
                    'is_del' => OIFCModel::$is_del_normal,
                    'status' => OIFCModel::$status_wait_validate
                ])->find();

            //加写锁

            //1.数据校验
            $val = Validate::rule([
                'platform_code|平台订单号' => 'require|length:0,255',
                'po_code|其他单号' => 'length:0,255',
                'platform_time|平台下单时间' => 'require|date',
                'sale_source|销售渠道' => 'require|length:0,255',
                'plat_code|平台商品编号' => 'require|length:0,255',
                'price|单价' => 'require|float',
                'num|数量' => 'require|number',
                'before_discount_all_price|优惠前总金额' => 'require|float',
                'activity_name|优惠活动名称' => 'length:0,255',
                'mode|收费模式' => 'require|length:0,255',
                'order_remark|订单备注' => 'length:0,255',
                'contactor|收货人' => 'require|length:0,255',
                'mobile|联系电话' => 'require|length:0,20',
                'addr|联系地址' => 'require|length:0,255',
            ]);

            if (!$val->check($rs)) {
                $order_import_from_c_db
                    ->where('id', $rs['id'])
                    ->update([
                        'updatetime' => date('Y-m-d H:i:s'),
                        'status' => OIFCModel::$status_wait_confirm,
                        'remark' => '数据格式不对，' . $val->getError()
                    ]);

                //解除写锁
                Db::commit();
                return false;
            }


            $extend_insert_data = [
                'order_import_from_c_id' => $rs['id'],
                'createrid' => 0,
                'creater' => 'system',
                'discount_reason' => $rs['activity_name']
            ];
            //查找extend的数据

            $good_platform_info = Db::name('good_platform')
                ->field('id,spuCode,platform_code,skuCode')
                ->where(['is_del' => 0])
                ->where('plat_code', $rs['plat_code'])
                ->find();
            if (!empty($good_platform_info)) {
                $extend_insert_data['spuCode'] = $good_platform_info['spuCode'];
                $extend_insert_data['platform_code'] = $good_platform_info['platform_code'];
            } else {
                $order_import_from_c_db
                    ->where('id', $rs['id'])
                    ->update([
                        'updatetime' => date('Y-m-d H:i:s'),
                        'status' => OIFCModel::$status_wait_confirm,
                        'remark' => '在商品平台表中查不到平台商品编号对应的spuCode'
                    ]);
                //解除写锁
                Db::commit();
                return false;
            }


            $good_info = Db::name('good')
                ->field('good_name,good_code,companyNo,supplierNo,is_stock,packing_fee,cert_fee,open_fee,cost_fee,mark_fee,demo_fee,cat_id')
                ->where(['is_del' => 0])
                ->where('spuCode', $extend_insert_data['spuCode'])
                ->find();
            if (!empty($good_info)) {
                $extend_insert_data['companyNo'] = $good_info['companyNo'];
                $extend_insert_data['supplierNo'] = $good_info['supplierNo'];
                $extend_insert_data['good_name'] = $good_info['good_name'];
            } else {
                $order_import_from_c_db
                    ->where('id', $rs['id'])
                    ->update([
                        'updatetime' => date('Y-m-d H:i:s'),
                        'status' => OIFCModel::$status_wait_confirm,
                        'remark' => '在上线商品表中查不到spuCode对应的商品信息'
                    ]);
                //解除写锁
                Db::commit();
                return false;
            }

            $order_import_from_c_extend_db->insertGetId($extend_insert_data);

            //判断是否库存品
            if ($good_info['is_stock'] == 1) {
                //库存品，查询可用库存数量
                $usable_stock = Db::name('good_stock')
                    ->where('spuCode', $extend_insert_data['spuCode'])
                    ->sum('usable_stock');

                if ($usable_stock < $rs['num']) {
                    $order_import_from_c_db
                        ->where('id', $rs['id'])
                        ->update([
                            'updatetime' => date('Y-m-d H:i:s'),
                            'status' => OIFCModel::$status_wait_relation,
                            'remark' => '库存不足'
                        ]);

                    //解除写锁
                    Db::commit();
                    return false;
                } else {
                    //扣减库存


                    //录入成功
                    $order_import_from_c_db
                        ->where('id', $rs['id'])
                        ->update([
                            'updatetime' => date('Y-m-d H:i:s'),
                            'status' => OIFCModel::$status_success,
                        ]);

                    $this->createSaleByImportSuccess($good_info, $rs, $extend_insert_data);

                }

            } else {
                //非库存品

                //录入成功
                $order_import_from_c_db
                    ->where('id', $rs['id'])
                    ->update([
                        'updatetime' => date('Y-m-d H:i:s'),
                        'status' => OIFCModel::$status_success,
                    ]);
                $orderCode = $this->createSaleByImportSuccess($good_info, $rs, $extend_insert_data);

                //生成采购单
                $cgd = [
                    "supplierNo" => $extend_insert_data['supplierNo'],
                    "companyNo" => $extend_insert_data['companyNo'],
                    "orderCode" => $orderCode,
                    "spuCode" => $extend_insert_data['spuCode'],
                    "skuCode" => $good_platform_info['skuCode'],
                    "good_name" => $extend_insert_data['good_name'],
                    "sale_price" => $rs['price'],
                    "total_fee" => $rs['price'] * $rs['num'],
                    "pakge_fee" => $good_info['packing_fee'],
                    "cert_fee" => $good_info['cert_fee'],
                    "open_fee" => $good_info['open_fee'],
                    "cost_fee" => $good_info['cost_fee'],
                    "mark_fee" => $good_info['mark_fee'],
                    "demo_fee" => $good_info['demo_fee'],
                    "good_num" => $rs['num'],
                    "good_type" => 1,//商品类型 1正常2赠品3样品
                    "order_type" => $good_info['is_stock'] == 1 ? 1 : 2,
                    "createrid" => $rs['createrid'],
                    "creater" => $rs['creater'],
                    'send_way' => 2
                ];
                $sale_controller = new Sale();

                $sale_controller->createCgd($cgd);
            }

            Db::commit();

            return true;

        } catch (\think\Exception $exception) {
            Db::rollback();
            return false;
        }

    }

    //【辅助1】获取线上商品详情
    public function getOnlineGoodInfoByPlatcode()
    {

        $plat_code = $this->request->post('plat_code', '', 'trim');

        if (!$plat_code) return error_show(1004, '平台商品编号不能为空');

        $res = Db::name('good_platform')
            ->alias('gp')
            ->field('gp.platform_code,gp.plat_code,g.*')
            ->where(['gp.plat_code' => $plat_code, 'gp.is_del' => 0])
            ->leftJoin('good g', 'g.spuCode=gp.spuCode AND g.is_del=0')
            ->find();

        if ($res) return app_show(0, '', $res);
        else return error_show(1005, '该平台商品编号不存在对应商品');

    }

    //【辅助2】接口2-获取销售渠道备选数据：入参所在平台Id,响应：销售渠道备选数据
    public function getSourceDataByPlatformId()
    {

        $platform_id = $this->request->post('platform_id/d', 0, 'trim');

        $list = Db::name('platform_source')
            ->field('id,platform_id,source')
            ->where(['is_del' => 0, 'platform_id' => $platform_id])
            ->order('id', 'desc')
            ->select()
            ->toArray();

        return app_show(0, '获取成功', $list);

    }


}